


#include <p18f4550.h>

#include <i2c.h>

#include "h/I2C.h"
#include "h/MemorisationsEEPROM.h"
#include "h/Thermometre.h"
#include "h/Affichages.h"
#include "h/Utilitaires.h"
#include "h/Parametres.h"



#pragma config BOR = ON
#pragma config MCLRE = OFF
#pragma config LVP = OFF
#pragma config PBADEN = OFF
#pragma config DEBUG = OFF
#pragma config FCMEN = OFF
#pragma config IESO = OFF
#pragma config PWRT = OFF
#pragma config WDT = OFF
#pragma config WDTPS = 1024
#pragma config LPT1OSC = OFF
#pragma config XINST = OFF
#pragma config STVREN = OFF 
#pragma config FOSC = INTOSCIO_EC



extern unsigned char g_Affichages[8];

extern int g_Minimale1;
extern int g_Maximale1;

extern int g_Minimale2;
extern int g_Maximale2;


extern unsigned char g_LimiteActualisations;



extern rom unsigned char ValeursPORTA[8];

extern rom unsigned char ValeursPORTB[8];



						
int g_Comptage = 0;

unsigned char g_Fonction = THERMOMETRE;

unsigned char g_ChoixAffichages = 0;



unsigned char g_ReinitialisationsExtremes = 0;

unsigned char g_EnregistrementsExtremes = 0;



unsigned char g_ChangementsComptage = 0;



void InitialisationMicrocontroleur(void)
{
   	OSCCON = 0xFF;

   	OSCTUNE = 0x10;

    CCP1CON = 0x00;

    ADCON1 = 0x0F; 
   
    T1CON = 0b10000000;

    INTCONbits.GIE = 0;

	INTCONbits.PEIE = 1;
	INTCON3bits.INT2IE = 1;
    PIE1bits.TMR1IE = 1;

	TRISA = 0xF0;
	TRISB = 0x0F;
	TRISC = 0x3B;
	TRISD = 0x00;
	TRISE = 0xFF;

	PORTA = 0x00;
	PORTB = 0x00;
	PORTC = 0x00;
	PORTD = 0x00;
	PORTE = 0x00;
}

void main(void)
{
	unsigned char k;
	
	unsigned char FonctionPrecedente = THERMOMETRE;
	
	unsigned char T1 = 0x00;

	unsigned char T2 = 0x00;

	unsigned char c;

	int Temperatures = 0;
		
	
	InitialisationMicrocontroleur();
		
		Delai(30);
		
		FonctionPrecedente = RecupererFonctionPrecedente();

		g_Fonction = AttribuerUneFonction(FonctionPrecedente);

		switch(g_Fonction)
		{
			case THERMOMETRE :
			{
				OpenI2C(MASTER, SLEW_OFF);

				SSPADD = 30;
	
				ConfigurerUnCapteur(ADRESSE_CAPTEUR_1);
				ConfigurerUnCapteur(ADRESSE_CAPTEUR_2);

				LED_MAXIMALE_MINIMALE = 0;

				LED_INTERIEURE = 0;
				LED_EXTERIEURE = 0;

				g_LimiteActualisations = 255;

				RecupererExtremes1(&g_Minimale1, &g_Maximale1);
				RecupererExtremes2(&g_Minimale2, &g_Maximale2);

				Read16nouvelle(ADRESSE_CAPTEUR_1, 0x00, (ADRESSE_CAPTEUR_1 | 0x01), &T1, &T2);
				Read16nouvelle(ADRESSE_CAPTEUR_2, 0x00, (ADRESSE_CAPTEUR_2 | 0x01), &T1, &T2);

				Delai(10000);

    			INTCONbits.GIE = 1;

				while(1)
				{
					switch(g_ChoixAffichages)
					{
						case AFFICHAGE_TEMPERATURES_MESUREES :
						{
							g_LimiteActualisations = 100;

							if(LireTemperature1(&T1, &T2) == 0x00)
							{
								g_Affichages[0] = ConvertirPourAffichage('E');
								g_Affichages[1] = 0x00;
								g_Affichages[2] = 0x00;
								g_Affichages[3] = 0x00;

								LED_INTERIEURE = 0;
							}
							else
							{
								MaximumMinimum1(T1, T2);
								AfficherUneTemperature(AFFICHAGE_TEMPERATURE_1, T1, T2);

								LED_INTERIEURE = 1;
							}
							
							if(LireTemperature2(&T1, &T2) == 0x00)
							{
								g_Affichages[4] = ConvertirPourAffichage('E');
								g_Affichages[5] = 0x00;
								g_Affichages[6] = 0x00;
								g_Affichages[7] = 0x00;

								LED_EXTERIEURE = 0;
							}
							else
							{
								MaximumMinimum2(T1, T2);
								AfficherUneTemperature(AFFICHAGE_TEMPERATURE_2, T1, T2);

								LED_EXTERIEURE = 1;
							}

							LED_MAXIMALE_MINIMALE = 0;

							break;
						}
						case AFFICHAGE_TEMPERATURES_EXTREMES_1 :
						{
							EntierVersOctets(g_Minimale1, &T1, &T2);
							AfficherUneTemperature(AFFICHAGE_TEMPERATURE_2, T1, T2);	/* Affichage de la temprature intrieure minimale. */

							EntierVersOctets(g_Maximale1, &T1, &T2);
							AfficherUneTemperature(AFFICHAGE_TEMPERATURE_1, T1, T2);	/* Affichage de la temprature intrieure maximale. */

							if((g_ReinitialisationsExtremes == 0) && (g_EnregistrementsExtremes == 1))
							{
								ConfigurerUnCapteur(ADRESSE_CAPTEUR_1);
								ConfigurerUnCapteur(ADRESSE_CAPTEUR_2);

								Read16nouvelle(ADRESSE_CAPTEUR_1, 0x00, (ADRESSE_CAPTEUR_1 | 0x01), &T1, &T2);
								Read16nouvelle(ADRESSE_CAPTEUR_2, 0x00, (ADRESSE_CAPTEUR_2 | 0x01), &T1, &T2);

								EnregistrerExtremes1(g_Minimale1, g_Maximale1);

								g_EnregistrementsExtremes = 0;
							}

							if(g_ReinitialisationsExtremes == 1)
							{
								ConfigurerUnCapteur(ADRESSE_CAPTEUR_1);
								ConfigurerUnCapteur(ADRESSE_CAPTEUR_2);
				
								Read16nouvelle(ADRESSE_CAPTEUR_1, 0x00, (ADRESSE_CAPTEUR_1 | 0x01), &T1, &T2);
								Read16nouvelle(ADRESSE_CAPTEUR_2, 0x00, (ADRESSE_CAPTEUR_2 | 0x01), &T1, &T2);

								ReinitialiserExtremes1();

								EnregistrerExtremes1(g_Minimale1, g_Maximale1);

								g_ReinitialisationsExtremes = 0;
							}

							LED_MAXIMALE_MINIMALE = 1;

							LED_INTERIEURE = 1;
							LED_EXTERIEURE = 0;

							break;
						}
						case AFFICHAGE_TEMPERATURES_EXTREMES_2 :
						{
							EntierVersOctets(g_Minimale2, &T1, &T2);
							AfficherUneTemperature(AFFICHAGE_TEMPERATURE_2, T1, T2);	/* Affichage de la temprature extrieure minimale. */

							EntierVersOctets(g_Maximale2, &T1, &T2);
							AfficherUneTemperature(AFFICHAGE_TEMPERATURE_1, T1, T2);	/* Affichage de la temprature extrieure maximale. */

							if((g_ReinitialisationsExtremes == 0) && (g_EnregistrementsExtremes == 1))
							{
								ConfigurerUnCapteur(ADRESSE_CAPTEUR_1);
								ConfigurerUnCapteur(ADRESSE_CAPTEUR_2);

								Read16nouvelle(ADRESSE_CAPTEUR_1, 0x00, (ADRESSE_CAPTEUR_1 | 0x01), &T1, &T2);
								Read16nouvelle(ADRESSE_CAPTEUR_2, 0x00, (ADRESSE_CAPTEUR_2 | 0x01), &T1, &T2);

								EnregistrerExtremes2(g_Minimale2, g_Maximale2);

								g_EnregistrementsExtremes = 0;
							}

							if(g_ReinitialisationsExtremes == 1)
							{
								ConfigurerUnCapteur(ADRESSE_CAPTEUR_1);
								ConfigurerUnCapteur(ADRESSE_CAPTEUR_2);

								Read16nouvelle(ADRESSE_CAPTEUR_1, 0x00, (ADRESSE_CAPTEUR_1 | 0x01), &T1, &T2);
								Read16nouvelle(ADRESSE_CAPTEUR_2, 0x00, (ADRESSE_CAPTEUR_2 | 0x01), &T1, &T2);

								ReinitialiserExtremes2();

								EnregistrerExtremes2(g_Minimale2, g_Maximale2);

								g_ReinitialisationsExtremes = 0;
							}

							LED_MAXIMALE_MINIMALE = 1;

							LED_INTERIEURE = 0;
							LED_EXTERIEURE = 1;

							break;
						}
						default :
						{

							break;
						}
					}

					for(c = 0; c < g_LimiteActualisations; c++)
					{
						for(k = 0; k < 8; k++)
						{
							PORTA = ValeursPORTA[k];
							PORTB = ValeursPORTB[k];

							PORTD = g_Affichages[k];

							Delai(30);

							PORTD = 0x00;
						}
					}
				}
				break;
			}
			case COMPTEUR :
			{
				LED_MAXIMALE_MINIMALE = 0;

				LED_INTERIEURE = 0;
				LED_EXTERIEURE = 0;
				
				g_Comptage = RecupererValeurComptage();

				if(g_Comptage >= (VALEUR_MAXIMALE_COMPTAGE + 1))
				{
					g_Comptage = VALEUR_MINIMALE_COMPTAGE;

					EnregistrerValeurComptage(VALEUR_MINIMALE_COMPTAGE);
				}

				g_Affichages[0] = ConvertirPourAffichage('C');
				g_Affichages[1] = ConvertirPourAffichage('p');
				g_Affichages[2] = (MASQUAGE_AFFICHAGE_VIRGULE | ConvertirPourAffichage('t'));
				g_Affichages[3] = 0x00;

				AfficherValeurComptage(g_Comptage);

    			INTCONbits.GIE = 1;

				while(1)
				{
					if(g_ChangementsComptage == 1)
					{
						EnregistrerValeurComptage(g_Comptage);
						
						AfficherValeurComptage(g_Comptage);
						
						g_ChangementsComptage = 0;
					}

					for(k = 0; k < 8; k++)
					{
						PORTA = ValeursPORTA[k];
						PORTB = ValeursPORTB[k];
						PORTD = g_Affichages[k];
						Delai(30);
						PORTD = 0x00;
					}
				}
				break;
			}
			default :
			{
				while(1)
				{
				
				}
				break;
			}
		}
	}



#pragma interrupt InterruptHandler   
void InterruptHandler (void)
{
	static unsigned char RestituerAffichageMesurees = 0;
	static unsigned char ReinitialisationCompteur = 0;

    if(INTCON3bits.INT2IE && INTCON3bits.INT2IF)
    {
		TRISC = 0x3B;

		PORTA = PORTA & ~(0x0F);
		PORTB = PORTB & ~(0xF0);

		PORTD = 0x00;

		Delai(300);

		switch(g_Fonction)
		{
			case THERMOMETRE :
			{
				if(BOUTON_POUSSOIR_MAXIMA_MINIMA == 1)
				{
					if(g_ChoixAffichages <= AFFICHAGE_TEMPERATURES_EXTREMES_1)
					{
						g_LimiteActualisations = 3;
						g_ChoixAffichages++;
						g_EnregistrementsExtremes = 1;

						RestituerAffichageMesurees = 250;

						T1CONbits.TMR1ON = 1;
					}
					else
					{
						g_ChoixAffichages = AFFICHAGE_TEMPERATURES_MESUREES;
					}
				}
				else
				{
					if(BOUTON_POUSSOIR_REINITIALISATIONS == 1)
					{
						if((g_ChoixAffichages == 1) || (g_ChoixAffichages == 2))
						{
							g_ReinitialisationsExtremes = 1;

							RestituerAffichageMesurees = 250;

							T1CONbits.TMR1ON = 1;	
						}
					}
				}	
				break;
			}
			case COMPTEUR :
			{
				if(BOUTON_POUSSOIR_MAXIMA_MINIMA == 1)
				{
					if(g_Comptage <= (VALEUR_MAXIMALE_COMPTAGE - 1))
					{
						g_ChangementsComptage = 1;
	
						g_Comptage++;
					}
				}
				else
				{
					if(BOUTON_POUSSOIR_REINITIALISATIONS == 1)
					{
						if(g_Comptage >= (VALEUR_MINIMALE_COMPTAGE + 1))
						{
							g_ChangementsComptage = 1;
	
							g_Comptage--;
						}
					}
					else
					{
						if(g_Comptage >= (VALEUR_MINIMALE_COMPTAGE + 1))
						{
							ReinitialisationCompteur = 10;

							T1CONbits.TMR1ON = 1;
						}
					}
				}
				break;
			}
			default :
			{
				break;
			}
		}

        INTCON3bits.INT2IF = 0;
    }
	if(PIE1bits.TMR1IE && PIR1bits.TMR1IF)
    {
		if(RestituerAffichageMesurees >= 2)
		{
			RestituerAffichageMesurees--;
		}
		if(RestituerAffichageMesurees == 1)
		{
			RestituerAffichageMesurees = 0;

			g_ChoixAffichages = AFFICHAGE_TEMPERATURES_MESUREES;
		}

		if(ReinitialisationCompteur >= 2)
		{
			ReinitialisationCompteur--;
		}
		if(ReinitialisationCompteur == 1)
		{
			ReinitialisationCompteur = 0;

			if((BOUTONS_POUSSOIRS_INFORMATIONS == 1) && (BOUTON_POUSSOIR_MAXIMA_MINIMA == 0) && (BOUTON_POUSSOIR_REINITIALISATIONS == 0))
			{
				g_Comptage = VALEUR_MINIMALE_COMPTAGE;

				g_ChangementsComptage = 1;
			}
		}

		if((RestituerAffichageMesurees == 0) && (ReinitialisationCompteur == 0))
		{
			T1CONbits.TMR1ON = 0;
		}
		PIR1bits.TMR1IF = 0;
	}
}

#pragma code InterruptVectorHigh = 0x08 
void InterruptVectorHigh(void)
{
    _asm
        goto InterruptHandler
    _endasm
}
 